%
% $Id: varda_server.pl,v 1.0 2008-04-20 23:05:55 lly Exp $
%
% Copyright 2008 by Łukasz Łysik
%
% VARDA server 
%
%
%     Copyright (C) 2006-9 by the HeKatE Project
%
%     VARDA has been develped by the HeKatE Project, 
%     see http://hekate.ia.agh.edu.pl
%
%     This file is part of VARDA.
%
%     VARDA is free software: you can redistribute it and/or modify
%     it under the terms of the GNU General Public License as published by
%     the Free Software Foundation, either version 3 of the License, or
%     (at your option) any later version.
%
%     VARDA is distributed in the hope that it will be useful,
%     but WITHOUT ANY WARRANTY; without even the implied warranty of
%     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%     GNU General Public License for more details.
%
%     You should have received a copy of the GNU General Public License
%     along with VARDA.  If not, see <http://www.gnu.org/licenses/>.
%


:- write('VARDA: $Id: varda_server.pl,v 1.0 2008-04-20 23:05:55 lly Exp $'),nl.


% tworzenie serwera

create_server(Port) :-
    tcp_socket(Socket),
    tcp_bind(Socket, Port),
    tcp_listen(Socket, 5),
    tcp_open_socket(Socket, AcceptFd, _),
    dispatch(AcceptFd).

:- dynamic(conn/3).

dispatch(AcceptFd) :-
    tcp_accept(AcceptFd, Socket, _Peer),
    fork(Pid),
    ( Pid == child
      -> tcp_open_socket(Socket, In, Out),
      assert(conn(Socket,In,Out)),
      handle_service(In, Out,Pid),
      close(In),
      close(Out),
      halt
    ; tcp_close_socket(Socket)
    ),
    dispatch(AcceptFd).

handle_service(In, Out,Pid) :-
	read(In,X),
	write('Otrzymano: '),
	write(X), nl,
	  ( X == 'list_deps'
	  ->
	    call_with_ret(list_deps(Ret),_),
	    write_to_socket(Ret),
	    handle_service(In,Out,Pid)
	  ;
	   ( X == 'list_props'
	   ->
	    call_with_ret(list_props(Ret),_),
	    write_to_socket(Ret),
	    handle_service(In,Out,Pid)
	    ;
	     ( X == 'list_finalizeables'
	     ->
	      call_with_ret(list_finalizeables(Ret),_),
	      write_to_socket(Ret),
	      handle_service(In,Out,Pid)
	      ;
	      ( X == 'list_splitables'
	      ->
	       call_with_ret(list_splitables(Ret),_),
	       write_to_socket(Ret),
	       handle_service(In,Out,Pid)
	      ;
	      ( X == 'list_items_from_svg'
	      -> 
	       sar_file('therm-model.dot'),
	       shell('dot -T svg therm-model.dot > therm-model.svg'),
	       call_with_ret(items_from_svg(Ret,'therm-model'),_),
	       write_to_socket(Ret),
	       handle_service(In,Out,Pid)
	      ;
	      ( X == 'list_coords_from_svg'
	      -> 
	       sar_file('therm-model.dot'),
	       shell('dot -T svg therm-model.dot > therm-model.svg'),
	       call_with_ret(coords_from_svg(Ret,'therm-model'),_),
	       write_to_socket(Ret),
	       handle_service(In,Out,Pid)
	       ;
	  Com = X,
	  call_with_ret(Com,Ans),
	  write_to_socket(Ans),
	  handle_service(In,Out,Pid)
	  )))))
	; write('Closing server..'),
	handle_service(In,Out,Pid)
	).

% zapisz do socketu
write_to_socket(Txt) :-
	conn(_,_,Out),
	write(Out,Txt),
	nl(Out),
	flush_output(Out).

% wykonaj polecenie ze zwroceniem rezultatu Yes/No
call_with_ret(end_of_file,Ret) :- Ret = 'No', !.
call_with_ret(C,Ret) :- C, Ret = 'Yes',!.
call_with_ret(_,Ret) :- Ret = 'No',!.

% listuj properties
list_prop(Ret) :- 
	ard_property(Ret),
	\+ ard_hist(Ret,_).

% listuj properties i zapisz do listy
list_props(Ret) :-
	bagof(X,list_prop(X),Ret).
	
% listuj finalizowalne
ard_finalizeable(Ret) :-
	list_prop(Ret),
	[_] = Ret.

% listuj finalizowalne i zapisz do listy
list_finalizeables(Ret) :-
	bagof(X,ard_finalizeable(X),Ret).
	
% listuj splitowalne
ard_splitable(Ret) :-
	list_prop(Ret),
	\+ [_] = Ret.
	
% listuj splitowalne i zapisz do listy
list_splitables(Ret) :-
	bagof(X,ard_splitable(X),Ret).
	
% listuj zależności	
list_deps(Ret) :-
	bagof([X-Y],ard_depend(X,Y),Ret).
	
%ard_finalize_2(X,Y) :-
%	ard_att_add_2(Y),
%	ard_finalize(X,Y).
%
%ard_att_add_2([H|T]) :-
%	ard_att_add(H),
%	ard_att_add_2(T).
%
%ard_att_add_2([]).

% splituje z tworzeniem properies

ard_split_2(Property,DependList) :-
	make_new_property_list(Property,[],DependList,NewPropertyList),
	ard_split(Property, NewPropertyList, DependList),!.

% tworzy liste properties	
make_new_property_list(Property,Start,[HDL|TDL],Ret3) :-
	[HDL1|[HDL2]] = HDL,
	check_and_add(Property,Start,HDL1,Ret1),
	check_and_add(Property,Ret1,HDL2,Ret2),
	make_new_property_list(Property,Ret2,TDL,Ret3).

make_new_property_list(_,A,_,A).	

check_and_add(Property,NewPropertyList,HDL,[HDL|NewPropertyList]) :-
	write('Checking: '), write(HDL), write(' - '), write(Property), nl,
	contains(HDL,Property),
	\+ contains(HDL,NewPropertyList),
	write('Adding: '), write(HDL),nl.

check_and_add(_,A,_,A).
	


contains([H|T],L) :-
	item_contains(H,L),
	contains(T,L).

contains([],_).


item_contains(A,[A|_]).

item_contains(A,[_|T]) :-
	item_contains(A,T).
	
item_contains(A,[A]).

test(X) :-
	make_new_property_list([day,month,today],[],[[[month],[season]],[[day,today],[operation]]],X).	

test2 :-
	test(X),!,write(X).
	